/**
  *  \file client/dlg-structure.h
  *  \brief Structure Building Dialog
  *
  *  (c) 2001-2013 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_DLG_STRUCTURE_H
#define PCC_V2_CLIENT_DLG_STRUCTURE_H

#include "client/actions.h"
#include "game/planet.h"
#include "ui/widgets/focusgroup.h"
#include "ui/widgets/button.h"

class GPlanetBuildStructuresAction;
class GPlanet;
class GUniverse;
class UICardGroup;

/** Structure Building Widget. Displays a structure with count, goals
    and cost, and allows the user to build or scrap it. */
class WPlanetStructureWidget : public UIWidget {
    GPlanetBuildStructuresAction&  action;
    TPlanetaryStructure            structure;
    SignalConnection               conn;
 public:
    WPlanetStructureWidget(GPlanetBuildStructuresAction& action,
                           TPlanetaryStructure structure);
    ~WPlanetStructureWidget();
    void add(int arg);
    void remove(int arg);

    void drawContent(GfxCanvas& can);
    bool handleEvent(const UIEvent& e, bool second_pass);
    void getLayoutInfo(LayoutInfo& info);
};

/** Structure Building Cost Display. Displays the cost of a
    GPlanetBuildStructuresAction. Normally, shows cash and supplies,
    but can also show Duranium amounts in addition. */
class WPlanetStructureCostWidget : public UIWidget {
    GPlanetBuildStructuresAction& action;
    bool                          with_duranium;
    SignalConnection              conn_action, conn_planet;
    int                           label_width, number_width, unit_width;
    static const char* const labels[];
    static const char* const units[];

    void computeLayout();
 public:
    WPlanetStructureCostWidget(GPlanetBuildStructuresAction& action,
                               bool with_duranium);
    ~WPlanetStructureCostWidget();

    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
};

/** Structure Building Effect Display. Displays the effects of
    building structures with respect to taxation and sensor visibility. */
class WPlanetStructureEffectWidget : public UIWidget {
    GPlanetBuildStructuresAction& action;
    SignalConnection              conn_action, conn_planet;
 public:
    WPlanetStructureEffectWidget(GPlanetBuildStructuresAction& action);
    ~WPlanetStructureEffectWidget();

    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
};

/** Structure Building Defense Effect Display. Displays the effects
    of defense posts, i.e. the weapon types and counts generated
    by the defense posts. */
class WPlanetDefenseEffectWidget : public UIWidget {
    GPlanetBuildStructuresAction& action;
    SignalConnection conn_action;

    void showLine(GfxContext& ctx, int dy, string_t name, int next, int have, int max);
 public:
    WPlanetDefenseEffectWidget(GPlanetBuildStructuresAction& action);
    ~WPlanetDefenseEffectWidget();

    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
};

/** Structure Building Screen Header. Displays a header in a similar
    style to control screen headers, which in addition indicates the
    currently-shown page. */
class WPlanetStructureHeader : public UIBaseComplexWidget {
    GPlanet&         planet;
    UICardGroup&     group;
    SignalConnection conn;
    UIButton         btn_left, btn_right;
 public:
    WPlanetStructureHeader(GPlanet& planet, UICardGroup& group);
    ~WPlanetStructureHeader();

    void drawContent(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& li);
    void onResize(GfxRect oldpos, bool by_layout);
};

class WPlanetStructureGroup : public UIFocusGroup {
    WPlanetStructureWidget psw;
    UIButton add_btn, sub_btn;
    TPlanetaryStructure structure;
 public:
    WPlanetStructureGroup(int id,
                          GPlanetBuildStructuresAction& action,
                          TPlanetaryStructure structure);
    ~WPlanetStructureGroup();
    void getLayoutInfo(LayoutInfo& info);
    void onResize(GfxRect old_size, bool by_layout);
    void drawContent(GfxCanvas& can);
};

void doPlanetBuildScreen(GPlanet& planet, int start_screen);

#endif
