/**
  *  \file game/planettransfer.cc
  *  \brief Planet Transfer
  */

#include "game/planettransfer.h"
#include "game/planet.h"
#include "game/error.h"
#include "util/nls.h"
#include "game/actions/preconditions.h"

GPlanetTransfer::GPlanetTransfer(GPlanet& planet)
    : planet(planet)
{
    setFlags(can_GetUnload | can_Sup2mc);
    planet.cargo_arbiter.incUseCount();
    mustBePlayed(planet);
}

GPlanetTransfer::~GPlanetTransfer()
{
    if (!isFinished()) {
        doCancel();
    }
    planet.cargo_arbiter.decUseCount();
}

int32_t
GPlanetTransfer::getCargoOnObject(GCargoType type)
{
    return planet.getCargo(type);
}

void
GPlanetTransfer::changeCargoOnObject(GCargoType type, int32_t amount)
{
    planet.changeCargoRaw(type, amount);
}

void
GPlanetTransfer::changeReservedCargo(GCargoType type, int32_t amount)
{
    planet.cargo_arbiter.changeReservedCargo(type, amount);
}

bool
GPlanetTransfer::canHaveCargo(GCargoType cargo_id)
{
    if (cargo_id == el_Fighters
        || (cargo_id >= el_Torps && cargo_id < el_Torps + NUM_TORPS))
    {
        if (!planet.hasBase())
            return false;
    }
    return true;
}

int32_t
GPlanetTransfer::getMaxCargo(GCargoType cargo_id)
{
    // FIXME: Host .31 torp safety
    /* the torp/ore limits are arbitrary, but should match Tim's TRN check. */
    if (cargo_id == el_Fighters)
        return 60; /* FIXME */
    else if (cargo_id >= el_Torps && cargo_id < el_Torps + NUM_TORPS)
        return 10000;
    else
        return 1000000000;
}

string_t
GPlanetTransfer::getName() const
{
    // FIXME: PCC 1.x says "$NAME starbase" or "planet $NAME" here.
    // should we use same wording?
    return planet.getName(GObject::PlainName);
}

int32_t
GPlanetTransfer::getDisplayOffset(GCargoType /*type*/)
{
    return 0;
}
