#!/usr/bin/perl -w
#
#  Name symbols using Borland C++ Map file
#
#  Use as
#    perl namesymbols.pl file.map < backtrace.txt
#  This provides names for all lines of the form 'eip=nnnnnn'.
#

use strict;

my %symbolsByAddress;     # address => name
my %sections;             # number => { start => address, length => size, name => name }

foreach (@ARGV) {
    my $mode = 'none';    # none, 'sections', 'symbols'
    open FILE, "< $_" or die "$_: $!";
    while (<FILE>) {
        if (/^ Start/) {
            $mode = 'sections';
        } elsif (/Publics by Name/) {
            $mode = 'symbols';
        } elsif ($mode eq 'sections' && /^ *([0-9A-F]+):([0-9A-F]+)\s+([0-9A-F]+)H\s+(\S+)/i) {
            $sections{hex($1)} = {
                start => hex($2),
                length => hex($3),
                name => $4
                };
        } elsif ($mode eq 'symbols' && /^ *([0-9A-F]+):([0-9A-F]+)\s+(.*)/i) {
            my $sec = hex($1);
            my $ofs = hex($2);
            my $name = $3;
            if (!defined($sections{$sec})) {
                warn "unknown section '$1' for symbol '$3'\n";
            } else {
                $symbolsByAddress{$ofs + $sections{$sec}{start}} = $name;
            }
        }
    }
    close FILE;
}

# Convert symbol list into array
my @symbols = sort {$a<=>$b} keys %symbolsByAddress;
#print join("\n", @symbols);
#exit;

while (<STDIN>) {
    s/eip=([0-9a-f]+)/"$& <" . nameSymbol(hex($1)) . ">"/ei;
    print;
}

sub nameSymbol {
    my $eip = shift;
    # Really simple method:
    my $slot = @symbols;
    while ($slot > 0 && $symbols[$slot-1] >= $eip) { --$slot }
    if ($eip >= $symbols[$slot-1]) {
        my $adr = $symbols[$slot-1];
        my $rv = $symbolsByAddress{$adr};
        if ($adr != $eip) {
            $rv .= " + " . ($eip - $adr);
        }
        $rv;
    } else {
        sprintf "unknown %d %x", $slot, $eip;
    }
}
